package zero1hd.rhythmbullet.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.FrameBuffer;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.viewport.ScreenViewport;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.SongListController;
import zero1hd.rhythmbullet.graphics.ui.pages.CreditsPage;
import zero1hd.rhythmbullet.graphics.ui.pages.MainPage;
import zero1hd.rhythmbullet.graphics.ui.pages.MoreOptionsPage;
import zero1hd.rhythmbullet.graphics.ui.pages.OptionsPage;
import zero1hd.rhythmbullet.audio.SongList;
import zero1hd.rhythmbullet.util.TransitionAdapter;

public class MainMenu extends ScreenAdapter implements TransitionAdapter {
	public Stage stage;
	private Vector3 targetPosition;
	
	private MainPage mainPage;
	private OptionsPage optionsPage;
	private CreditsPage creditsPage;
	private MoreOptionsPage moreOptionsPage;
	
	private RhythmBullet core;
	
	private SongListController sc;
	private float lerpAlpha;
	
	private ShaderProgram gaussianBlurShader;
	private ShaderProgram brightFilterShader;
	private ShaderProgram combineShader;
	private FrameBuffer lightFilterBuffer;
	private FrameBuffer normalBuffer;
	private FrameBuffer hBlur, vBlur;
	private TextureRegion fboRegion;
	private int fboSize;
	
	private int blurlvl = 2;
	
	public MainMenu(final RhythmBullet core) {
		this.core = core;
		stage = new Stage(new ScreenViewport());
		targetPosition = new Vector3(stage.getCamera().position);
		
		SongList songList = new SongList();
		songList.setSearchPath(core.getPrefs().getString("music dir"));
		songList.refresh();
		
		sc = new SongListController(songList, core.getPrefs());
		sc.setAutoPlay(true);
		sc.setShuffle(true);
		
		loadShaders();
		
		postTransition();
	}
	
	public void loadShaders() {
		if (core.getPrefs().getBoolean("glow shader")) {
			Gdx.app.debug("Shader", "using glow shader");
			brightFilterShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/bright_filter.fsh"));
			if (!brightFilterShader.isCompiled()) {
				System.err.println(brightFilterShader.getLog());
				System.exit(0);
			}
			if (brightFilterShader.getLog().length() != 0) {
				System.out.println(brightFilterShader.getLog());
			}
				
			gaussianBlurShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/gaussian_blur.fsh"));
			if (!gaussianBlurShader.isCompiled()) {
				System.err.println(gaussianBlurShader.getLog());
				System.exit(0);
			}
			if (gaussianBlurShader.getLog().length() != 0) {
				System.out.println(gaussianBlurShader.getLog());
			}
			
			combineShader = new ShaderProgram(Gdx.files.internal("shaders/basic.vsh"), Gdx.files.internal("shaders/combine.fsh"));
			if (!combineShader.isCompiled()) {
				System.err.println(combineShader.getLog());
				System.exit(0);
			}
			if (combineShader.getLog().length() != 0) {
				System.out.println(combineShader.getLog());
			}
			
			
			if (Gdx.graphics.getWidth() < 1024) {
				fboSize = 2048;
			} else if (Gdx.graphics.getWidth() < 2048) {
				fboSize = 4096;
			} else {
				fboSize = 4096;
			}
			
			lightFilterBuffer = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);
			normalBuffer = new FrameBuffer(Format.RGBA8888, fboSize, fboSize, false);
			hBlur = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);
			vBlur = new FrameBuffer(Format.RGBA8888, fboSize/2, fboSize/2, false);
			
			fboRegion = new TextureRegion();
			
			combineShader.begin();
			combineShader.setUniformi("u_texture1", 1);
			combineShader.end();
			
			vBlur.getColorBufferTexture().bind(1);
			
			Gdx.gl.glActiveTexture(GL20.GL_TEXTURE0);
		}
		
		ShaderProgram.pedantic = false;
	}
	
	@Override
	public void render(float delta) {
		Gdx.gl.glClearColor(0.212f, 0.094f, 0f, 1f);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		stage.act(delta);
		blurlvl = 5;
		
		if (gaussianBlurShader != null) {
			
			//Begin drawing a normal version of screen
			normalBuffer.begin();
			Gdx.gl.glClearColor(0.212f, 0.094f, 0f, 1f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			stage.draw();
			normalBuffer.end();
			//END
			
			//Begin light filtering
			lightFilterBuffer.begin();
			Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
			Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
			stage.getBatch().setShader(brightFilterShader);
			stage.getBatch().begin(); //BATCH STARTS HERE
			stage.getRoot().draw(stage.getBatch(), 1f);
			stage.getBatch().flush();
			lightFilterBuffer.end();
			
			//Horizontal gaussian blur
			hBlur.begin();
			fboRegion.setRegion(lightFilterBuffer.getColorBufferTexture());
			stage.getBatch().setShader(gaussianBlurShader);
			gaussianBlurShader.setUniformi("horizontal", 1);
			stage.getBatch().draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
			stage.getBatch().flush();
			hBlur.end();
			
			//Vertical gaussian blur
			vBlur.begin();
			fboRegion.setRegion(hBlur.getColorBufferTexture());
			stage.getBatch().setShader(gaussianBlurShader);
			gaussianBlurShader.setUniformi("horizontal", 0);
			stage.getBatch().draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
			stage.getBatch().flush();
			vBlur.end();
			
			for (int i = 0; i < blurlvl; i++) {
				//Horizontal gaussian blur
				hBlur.begin();
				fboRegion.setRegion(vBlur.getColorBufferTexture());
				stage.getBatch().setShader(gaussianBlurShader);
				gaussianBlurShader.setUniformi("horizontal", 1);
				stage.getBatch().draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
				stage.getBatch().flush();
				hBlur.end();
				
				//Vertical gaussian blur
				vBlur.begin();
				fboRegion.setRegion(hBlur.getColorBufferTexture());
				stage.getBatch().setShader(gaussianBlurShader);
				gaussianBlurShader.setUniformi("horizontal", 0);
				stage.getBatch().draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
				stage.getBatch().flush();
				vBlur.end();
			}
			
			//Draw everything to screen
			stage.getBatch().setShader(combineShader);
			fboRegion.setRegion(normalBuffer.getColorBufferTexture());
			fboRegion.flip(false, true);
			stage.getBatch().draw(fboRegion, 0f, 0f, stage.getWidth(), stage.getHeight());
			stage.getBatch().setShader(null);
			stage.getBatch().end(); //BATCH ENDS HERE
		} else {
			stage.draw();
		}
        
		if (stage.getCamera().position.x != targetPosition.x || stage.getCamera().position.y != targetPosition.y) {
			stage.getCamera().position.lerp(targetPosition, lerpAlpha);
		}
		
		super.render(delta);
	}
	
	@Override
	public void preTransition() {
		stage.clear();
		mainPage.dispose();
		optionsPage.dispose();
		creditsPage.dispose();
		moreOptionsPage.dispose();
	}
	
	@Override
	public void postTransition() {
		mainPage = new MainPage(core, targetPosition, sc);
		mainPage.setPosition(0, 0);
		stage.addActor(mainPage);
		
		//End main menu
		moreOptionsPage = new MoreOptionsPage(core, targetPosition);
		
		optionsPage = new OptionsPage(core, targetPosition, moreOptionsPage, sc);
		optionsPage.setPosition(Gdx.graphics.getWidth(), 0);
		stage.addActor(optionsPage);
		
		creditsPage = new CreditsPage(core.getDefaultSkin());
		creditsPage.setPosition(0, Gdx.graphics.getHeight());
		stage.addActor(creditsPage);
		
		moreOptionsPage.setPosition(1f*Gdx.graphics.getWidth(), -1f*Gdx.graphics.getHeight());
		stage.addActor(moreOptionsPage);
		
		stage.addListener(new InputListener() {
			@Override
			public boolean keyUp(InputEvent event, int keycode) {
				if (keycode == Keys.ESCAPE) {
					stage.unfocusAll();
					if (targetPosition.x != 0.5f*Gdx.graphics.getWidth() || targetPosition.y != 0.5f*Gdx.graphics.getHeight()) {
						targetPosition.x = 0.5f*Gdx.graphics.getWidth();
						targetPosition.y = 0.5f*Gdx.graphics.getHeight();
					}
					moreOptionsPage.controlUnselect();
				}
				return super.keyUp(event, keycode);
			}
		});	
		
		stage.addListener(new ClickListener() {
			@Override
			public void clicked(InputEvent event, float x, float y) {
				if (stage.hit(x, y, true) == null) {
					stage.unfocusAll();
					moreOptionsPage.controlUnselect();
				}
				super.clicked(event, x, y);
			}
		});
		
		Gdx.app.debug("Post Transition", "Beginning screen setup for Main menu.");
	}
	
	@Override
	public void show() {
		Gdx.input.setInputProcessor(stage);
		sc.play();
		calcLerpAlpha(Gdx.graphics.getWidth());
		super.show();
	}
	
	@Override
	public void hide() {
		saveAll();
		super.hide();
	}
	
	public void saveAll() {
		if (optionsPage != null) {
			optionsPage.saveOptions(core.getPrefs());
			moreOptionsPage.getGraphicsSettings().save(core.getPrefs());
			core.getPrefs().flush();
		}
	}
	
	@Override
	public void resize(int width, int height) {
		stage.getViewport().update(width, height, false);
		targetPosition.x = width/2;
		targetPosition.y = height/2;
		calcLerpAlpha(width);
		super.resize(width, height);
	}
	
	@Override
	public void dispose() {
		stage.dispose();
		super.dispose();
	}
	
	private void calcLerpAlpha(int width) {
		if (width >= 3835) {
			lerpAlpha = 0.005f;
		} else {
			lerpAlpha = 0.25f;
		}
	}
}
